//	File:	rfr.v	  					
//	module name: rfr						
//	Description:	A generic register file of 4 16-bit registers, 3 read and 2 write ports.      
//			It has a synchronized reset control that sets all registers to $0000.	      
//      Author: Jonathan Tong


//*******************************************************************************
//										    
//	module:	rfr								 
//   
//*******************************************************************************


module rfr (
	reset,
	in1,
	in2,
	raddr1,
	raddr2,
	waddr1,
	waddr2,
	write1,
	write2,
	out1,	
	out2,	
	Clk
	);

/*===================    PARAMETERS   ================================*/

parameter dpname = "rfr_dp";

/*============	I/O direction	================================================*/

input reset;
input [15:0] in1;
input [15:0] in2;
input [1:0] raddr1;
input [1:0] raddr2;
input [1:0] waddr1;
input [1:0] waddr2;
input write1;
input write2;

output [15:0] out1;
output [15:0] out2;


input Clk;





/*============		I/O type	================================================*/

wire reset;				/* synchronized reset */
wire reset_inv;				/* inversion of reset signal */

wire [15:0] in1;		/* input data bus port 1 */
wire [15:0] in2;		/* input data bus port 2 */

wire [1:0] raddr1;		/* read address port 1 */
wire [1:0] raddr2;		/* read address port 2 */

wire [1:0] waddr1;		/* write address port 1 */
wire [1:0] waddr2;		/* write address port 2 */

wire write1;			/* synchronized write enable signal for incoming data on port 1 */
wire write2;			/* synchronized write enable signal for incoming data on port 2 */

wire muxen1;              /* input mux enable 1 */
wire muxen2;              /* input mux enable 2 */
wire muxen3;              /* input mux enable 3 */
wire muxen4;              /* input mux enable 4 */


wire [15:0] regin1;              /* input to register 1 */
wire [15:0] regin2;              /* input to register 2 */
wire [15:0] regin3;              /* input to register 3 */
wire [15:0] regin4;		 /* input to register 4 */

wire [15:0] regout1;              /* outtput of register 1 */
wire [15:0] regout2;              /* output of register 2 */
wire [15:0] regout3;              /* input to register 3 */
wire [15:0] regout4;	          /* input to register 4 */

wire we1;              /* write enable to register 1 */
wire we2;              /* write enable to register 2 */
wire we3;              /* write enable to register 3 */
wire we4;	       /* write enable to register 4 */

wire we1_inv;              /* inversion of write enable to register 1 */
wire we2_inv;              /* inversion of write enable to register 2 */
wire we3_inv;              /* inversion of write enable to register 3 */
wire we4_inv;	           /* inversion of write enable to register 4 */

wire [3:0] decodeout1;   /* output of the first decoder */
wire [3:0] decodeout2;   /* output of the second decoder */

wire [15:0] out1;		/* output data bus port 1 */
wire [15:0] out2;		/* output data bus port 2 */



wire Clk;

/*=======    Cascade/DesignWare module instantiation      ===============================*/

`ifdef DW
   DW01_mux_any #(32, 1, 16) inmux1 ({in2, in1}, muxen1, regin1);
`else   
   mux2 #(16,1,dpname) inmux1(
	.IN0(in1),
	.IN1(in2),
	.S0(muxen1),
	.Y(regin1)
	);
`endif 
   
`ifdef DW
   DW01_mux_any #(32, 1, 16) inmux2 ({in2, in1}, muxen2, regin2);
`else    
   mux2 #(16,1,dpname) inmux2(
	.IN0(in1),
	.IN1(in2),
	.S0(muxen2),
	.Y(regin2)
	);
`endif

`ifdef DW
   DW01_mux_any #(32, 1, 16) inmux3 ({in2, in1}, muxen3, regin3);
`else     
   mux2 #(16,1,dpname) inmux3(
	.IN0(in1),
	.IN1(in2),
	.S0(muxen3),
	.Y(regin3)
	);
`endif

`ifdef DW
   DW01_mux_any #(32, 1, 16) inmux4 ({in2, in1}, muxen4, regin4);
`else     
   mux2 #(16,1,dpname) inmux4(
	.IN0(in1),
	.IN1(in2),
	.S0(muxen4),
	.Y(regin4)
	);
`endif 
   
`ifdef DW
   assign reset_inv = ~reset; 
   // don't need we1_inv, we2_inv, we3_inv, and we4_inv  when using DesignWare components
`else
   inv #(1,0) reset_inverter(
	reset, reset_inv);

   inv #(1,0) we1_inverter(
	we1, we1_inv);

   inv #(1,0) we2_inverter(
	we2, we2_inv);

   inv #(1,0) we3_inverter(
	we3, we3_inv);

   inv #(1,0) we4_inverter(
	we4, we4_inv);
`endif 
   
`ifdef DW   
   DW03_reg_s_pl #(16, 0) reg1 (regin1, Clk, reset_inv, we1, regout1);   
`else   
   dffh_c #(16,1,dpname) reg1 (
	.CLK(Clk),
	.D(regin1),
	.HOLD(we1_inv),
	.CLR(reset_inv),
	.Q(regout1)
	);
`endif

`ifdef DW   
   DW03_reg_s_pl #(16, 0) reg2 (regin2, Clk, reset_inv, we2, regout2);   
`else 
   dffh_c #(16,1,dpname) reg2 (
	.CLK(Clk),
	.D(regin2),
	.HOLD(we2_inv),
	.CLR(reset_inv),
	.Q(regout2)
	);
`endif

`ifdef DW   
   DW03_reg_s_pl #(16, 0) reg3 (regin3, Clk, reset_inv, we3, regout3);   
`else    
  dffh_c #(16,1,dpname) reg3 (
	.CLK(Clk),
	.D(regin3),
	.HOLD(we3_inv),
	.CLR(reset_inv),
	.Q(regout3)
	);
`endif

`ifdef DW   
   DW03_reg_s_pl #(16, 0) reg4 (regin4, Clk, reset_inv, we4, regout4);   
`else    
   dffh_c #(16,1,dpname) reg4 (
	.CLK(Clk),
	.D(regin4),
	.HOLD(we4_inv),
	.CLR(reset_inv),
	.Q(regout4)
	);
`endif
 
`ifdef DW
   DW01_mux_any #(64, 2, 16) outmux1
       ({regout4, regout3, regout2, regout1}, raddr1, out1);  
`else   
   mux4 #(16,1,dpname) outmux1 (
	.IN0(regout1),
	.IN1(regout2),
	.IN2(regout3),
	.IN3(regout4),
	.S0(raddr1[0]),
	.S1(raddr1[1]),
	.Y(out1)
	);
`endif 
   
`ifdef DW
   DW01_mux_any #(64, 2, 16) outmux2
       ({regout4, regout3, regout2, regout1}, raddr2, out2);  
`else     
   mux4 #(16,1,dpname) outmux2 (
	.IN0(regout1),
	.IN1(regout2),
	.IN2(regout3),
	.IN3(regout4),
	.S0(raddr2[0]),
	.S1(raddr2[1]),
	.Y(out2)
	);
`endif 

// decoders for the 2 write ports   
mydecoder decoder1 (
	.SEL(waddr1),
	.Y(decodeout1)
	);

mydecoder decoder2 (
	.SEL(waddr2),
	.Y(decodeout2)
	);


assign muxen1 = write2 & decodeout2[0];
assign muxen2 = write2 & decodeout2[1];
assign muxen3 = write2 & decodeout2[2];
assign muxen4 = write2 & decodeout2[3];


assign we1 = (write1 & decodeout1[0]) | muxen1;
assign we2 = (write1 & decodeout1[1]) | muxen2;
assign we3 = (write1 & decodeout1[2]) | muxen3;
assign we4 = (write1 & decodeout1[3]) | muxen4;


endmodule	/* end module rf */


