/*	File:	bus_switch.v	  					        */

/*	module name: bus_switch						        */

/*	Description: The internal DATA bus switch for the CMU DSP core.	*/

/*==============================================================================*/
`ifdef SIM
   `include "../cmudsp/structural/globals.v"
   `include "../cmudsp/structural/ctl_def.v"
`else
   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"
`endif

/************************************************************************/
/*									*/
/*	module:	bus_switch						*/
/*					  			      	*/
/* scan chain: scan_in -> immediate -> pdb2 -> pdb3 -> scan_out         */
/************************************************************************/

module bus_switch (
	reset,
	PDB,
	GDB,
	XDB,
	YDB,
	REPEAT,
	immediate,
	Clk,
	scan_in, test,
	scan_out
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

   input	    reset;
   input [`databus] PDB;

   inout [`databus] GDB;

   inout [`databus] XDB;
   inout [`databus] YDB;

   input	    REPEAT;
   input  immediate;

   input  Clk;

   input  test;
   input  scan_in;
   output scan_out;


/*=========================================================================*/
/*============		I/O type	===================================*/
/*=========================================================================*/

   wire		   reset;
   wire [`databus] PDB;
   wire [`databus] GDB;		/* GDB bus	*/
   wire [`databus] XDB;		/* XDB bus	*/
   wire [`databus] YDB;		/* YDB bus	*/

   wire		   immediate;
		   
   wire		REPEAT;	/* REP instruction signal coming from the PCU */

   wire		   Clk;

   wire		   test;
   wire		   scan_out;


/*========================================================================*/
/*===========	Internal Nets	==========================================*/
/*========================================================================*/

   wire [`databus] pdb2;      /* an instruction inside stage 2 (Decode)	*/
   wire [`databus] pdb3;      /* an instruction inside stage 3 (Execute) */
   wire		   immediate_3;	     /* immediate signal delayedto stage 3 */

   /* enable signals for each set of tristate buffers */
   wire		   xdb_to_gdb;
   wire		   xdb_to_ydb;
   wire		   ydb_to_gdb;
   wire		   ydb_to_xdb;
   wire		   gdb_to_xdb;
   wire		   gdb_to_ydb;
   wire		   pdb_to_gdb;
   

/*-------------------------------------------------------------------*/
/* immediate is a flag  - when true signals that a source's absolute */
/* memory address or an immediate data is used.	*/
/* Such a use required TWO instruction words:   */
/* The instruction and the absolute address/immediate data.	*/
/*---------------------------------------------------------------*/


/*=========================================================================*/
/*									   */
/*	Processes				Processes                  */
/*				  					   */
/*=========================================================================*/

bs_control bs_control(
		   test,
                   immediate_3,
		   pdb3,
		   xdb_to_gdb,
		   xdb_to_ydb,
		   ydb_to_gdb,
		   ydb_to_xdb,
		   gdb_to_xdb,
		   gdb_to_ydb,
		   pdb_to_gdb
	);

/*======================================================================*/
/*	  					 			*/
/*	GDB  XDB  YDB  buses			      			*/
/*								      	*/
/*======================================================================*/

/* assigning data to the GDB, XDB and YDB buses */

   /* enable signals for each set of tristate buffers must be invered*/
   wire		    xdb_to_gdb_inv;
   wire		    xdb_to_ydb_inv;
   wire		    ydb_to_gdb_inv;
   wire		    ydb_to_xdb_inv;
   wire		    gdb_to_xdb_inv;
   wire		    gdb_to_ydb_inv;
   wire		    pdb_to_gdb_inv;

`ifdef DW
   assign GDB = xdb_to_gdb ? XDB : 24'bz;
   assign GDB = ydb_to_gdb ? YDB : 24'bz;
   assign GDB = pdb_to_gdb ? pdb2 : 24'bz;
   assign XDB = ydb_to_xdb ? YDB : 24'bz;
   assign XDB = gdb_to_xdb ? GDB : 24'bz;
   assign YDB = xdb_to_ydb ? XDB : 24'bz;
   assign YDB = gdb_to_ydb ? GDB : 24'bz;
`else 
   inv #(1,0) xdb_to_gdb_inverter(xdb_to_gdb, xdb_to_gdb_inv);
   inv #(1,0) ydb_to_gdb_inverter(ydb_to_gdb, ydb_to_gdb_inv);
   inv #(1,0) pdb_to_gdb_inverter(pdb_to_gdb, pdb_to_gdb_inv);
   inv #(1,0) ydb_to_xdb_inverter(ydb_to_xdb, ydb_to_xdb_inv);
   inv #(1,0) gdb_to_xdb_inverter(gdb_to_xdb, gdb_to_xdb_inv);
   inv #(1,0) xdb_to_ydb_inverter(xdb_to_ydb, xdb_to_ydb_inv);
   inv #(1,0) gdb_to_ydb_inverter(gdb_to_ydb, gdb_to_ydb_inv);

   tribuf #(24, 0) GDB_TRISTATE0 (xdb_to_gdb_inv, XDB, GDB);
   tribuf #(24, 0) GDB_TRISTATE1 (ydb_to_gdb_inv, YDB, GDB);
   tribuf #(24, 0) GDB_TRISTATE2 (pdb_to_gdb_inv, pdb2, GDB);
   tribuf #(24, 0) XDB_TRISTATE0 (ydb_to_xdb_inv, YDB, XDB);
   tribuf #(24, 0) XDB_TRISTATE1 (gdb_to_xdb_inv, GDB, XDB);
   tribuf #(24, 0) YDB_TRISTATE0 (xdb_to_ydb_inv, XDB, YDB);
   tribuf #(24, 0) YDB_TRISTATE1 (gdb_to_ydb_inv, GDB, YDB);
`endif


/*======================================================================*/
/*				       					*/
/*	Stage 2 (Decode)		    Stage 2	         	*/
/*		 							*/
/*======================================================================*/

wire [`databus] pdb2_in;	
wire reset_inv;

  // assign immediate = 1'b0;
`ifdef DW
`else
   inv #(1,0) reset_inverter(reset,reset_inv);
`endif 
   
assign pdb2_in = (reset) ? `NOP : PDB;

`ifdef DW
   DW03_reg_s_pl #(24, 0) BS_PDB2_DFF (pdb2_in, Clk, ~reset, ~REPEAT, pdb2); 
`else 
   dffhs_c #(24,0) BS_PDB2_DFF(Clk, reset_inv, pdb2_in, REPEAT,  immediate_3, test, pdb2);
`endif

/*=================================================================*/
/*	     							   */
/*	Stage 3 (Execute)			       		   */
/*			        				   */
/*=================================================================*/
`ifdef DW
   DW03_reg_s_pl #(1, 0) IMMEDIATE_3 (immediate, Clk, ~reset, 1'b1, immediate_3);
   DW03_reg_s_pl #(24, 0) BS_PDB3_DFF (pdb2, Clk, ~reset, 1'b1, pdb3); 
`else 
   dffs_c #(1,0) IMMEDIATE_3(Clk, reset_inv, immediate,  scan_in, test, immediate_3);

   dffs_c #(24,0) BS_PDB3_DFF(Clk, reset_inv, pdb2, pdb2[23], test, pdb3);
`endif

assign scan_out = pdb3[23];
       
endmodule	/* bus_switch */
