/****************************************************************************
 *	File:	agu_structural.v
 *	module name: agu_structural
 *
 *	Description:	The structural AGU, made out of only instantiated modules.
 *
 *	Author: Nitzan Weinberg	, Jonathan Tong
 *
 * scan chain:
 *    instantiated unit to unit; bit size in parens.
 *
 *    scanin -> agu_control_wrapper -> PDB2_DFF (24) -> PDB3_DFF (24) ->
 *    XAB_DFF (16) -> XWRITE_DFF (1) -> XREAD_DFF (1) -> YAB_DFF (16) ->
 *    YWRITE_DFF (1) -> YREAD_DFF (1) -> PAB_DFF (16) -> scanout
 ****************************************************************************/


// epoch set_attribute FIXEDBLOCK = 1
/*========================================================================*/




    `include "./verilog/globals.v"
    `include "./verilog/ctl_def.v"


/************************************************************************/
/*									*/
/*	module:	agu_structural						*/
/*									*/
/************************************************************************/

/* The following two modules are identified as fixed blocks modules */


module agu_structural (
	reset,
	PDB,
	GDB,
	REPEAT,
	PAB,
	XAB,
	YAB,
	XWRITE,
	XREAD,
	YWRITE,
	YREAD,
	immediate,
	scanin, test, scanout,
	Clk_0_rfm, Clk_1_rfn, Clk_2_rfr, Clk_3_ctrl, Clk_4_ctrl, Clk_5_misc, Clk_6_misc
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input  reset;
   
input [`databus] PDB;

inout [`databus] GDB;

input REPEAT;

   input	     scanin,test;
   output	     scanout;

   inout [`addrbus] PAB;
   output [`addrbus] XAB;
   output [`addrbus] YAB;
   output	     XWRITE;
   output	     XREAD;
   output	     YWRITE;
   output	     YREAD;
   output	     immediate;  // stage 3, for use by other units
    

input Clk_0_rfm;
input Clk_1_rfn;
input Clk_2_rfr;
input Clk_3_ctrl;
input Clk_4_ctrl;
input Clk_5_misc;
input Clk_6_misc;
   

/*==============================================================================*/
/*============		I/O type	================================================*/
/*=============================================================================*/

   wire		     reset;

   wire [`databus]   PDB;		/* Program data bus (instruction) */
   wire [`databus]   GDB;		/* General internal data bus */


   wire [`addrbus]   PAB;		/* Program address bus */
   wire [`addrbus]   XAB;		/* X data memory address bus */
   wire [`addrbus]   YAB;		/* Y data memory address bus */

   wire		     XWRITE;	/* X data memory write enable. 
				 * When TRUE, the core is driving XDB */
   wire		     XREAD;	/* X data memory read enable.  
				 * When TRUE, the X memory is driving XDB */
   wire		     YWRITE;	/* Y data memory write enable. 
				 * When TRUE, the core is driving YDB */
   wire		     YREAD;	/* Y data memory read enable.  
				 * When TRUE, the Y memory is driving YDB */

   wire		     immediate;  /* immediate flag.  High when stage 2 
				  * instruction is immediate data */
   

/*=============================================================================*/
/*===========	Internal Nets	===============================================*/
/*=============================================================================*/

/* input from the PCU */

wire REPEAT;


/*----------------------------------------------------------------------------------*/
/* fetched instruction coming from the PDB, and passed on along the pipeline stages */
/*----------------------------------------------------------------------------------*/

wire  [`databus] pdb2;	/* for pipeline stage 2 */
wire  [`databus] pdb3;	/* for pipeline stage 3 */


/*-------------------------------------------------------*/
/* X, Y and P memory address buses - internal to the AGU */
/*-------------------------------------------------------*/

wire [`addrbus] xab;						/* X address bus - from stage 2 (Decode) */

wire [2:0] xab_select;						/* select the input into xab  -  coming ONLY from stage 2 */




wire [`addrbus] yab;						/* Y address bus - from stage 2 (Decode) */

wire [2:0] yab_select;						/* select the input into yab  -  coming ONLY from stage 2 */


wire [`addrbus] PAB_2;						/* P Address bus - from stage 2 (Decode) */

wire [`addrbus] PAB_reg_out;	

wire [1:0] PAB_2_select;					/* select the input into PAB_2  -  coming ONLY from stage 2 */


/*------------------------------------------------------*/
/* X, Y memory read/write control - internal to the AGU */
/*------------------------------------------------------*/

wire xwrite;			/* write enable to the X data memory - from stage 2 (Decode) */

wire xread;				/* read enable from the X data memory - in stage 2 (Decode) */

wire ywrite;			/* write enable to the Y data memory - from stage 2 (Decode) */

wire yread;				/* read enable from the Y data memory - in stage 2 (Decode) */

wire PABwrite_2;		/* write program address bus - in stage 2 */

wire Xwrite;			/* The input to the write-enable XWRITE FF */
wire Xread;				/* The input to the read-enable  XREAD FF */

wire Ywrite;			/* The input to the write-enable YWRITE FF */
wire Yread;				/* The input to the read-enable  YREAD FF */


/*---------------------------------------------------------------------*/
/* The GDB output data bus and drive-control from the agu logic unit   */
/*---------------------------------------------------------------------*/
 
wire [`addrbus] gdb_out;					/* the *16-bit* AGU gdb output which is the input to the GDB_out register - from stage 2 */

wire [2:0] gdb_out_select;					/* select the input into gdb_out  -  coming ONLY from stage 2 */

wire gdb_write;								/* GDB_out write control - from stage 2 */

wire GDB_write;								/* FF - a write enable to the GDB bus */


/*--------------------------------------------------------------------------------------------------*/
/*	atomic flag and atomic-write enable																*/
/*																									*/
/*	The flag stays true as long as an atomic instrucion is being executed (BCLR, BSET, STST).		*/
/*	It prevents any other instruction from accessing the address and data buses that are			*/
/*	being used by the ongoing atomic instruction. Other requests for these buses are ignored		*/
/*	while the atomic instruction is being processed. This includes a newer atomic instruction		*/
/*	that arrives after the one that is still being executed.										*/
/*--------------------------------------------------------------------------------------------------*/

wire Xatomic;			/* Flag - signals that an atomic instruction that is using the XAB and XDB buses is in progress. */
wire xread_atomic;		/* read enable from the X data memory - generated by an atomic instruction in stage 2 */
wire xwrite_atomic;		/* write enable to the X data memory - generated by an atomic instruction in stage 3 */

wire Yatomic;			/* Flag - signals that an atomic instruction that is using the YAB and YDB buses is in progress. */
wire yread_atomic;		/* read enable from the Y data memory - generated by an atomic instruction in stage 2 */
wire ywrite_atomic;		/* write enable to the Y data memory - generated by an atomic instruction in stage 3 */



/*--------------------------------------------------------------------------------------------------*/
/* address alu control signals and output															*/
/*--------------------------------------------------------------------------------------------------*/

/*------*/
/* High */
/*------*/

wire addr_alu_H_N_or_1;				/* (addr_alu_H_N_or_1 = 0) => (choose Nn)		*/
									/* (addr_alu_H_N_or_1 = 1) => (choose 1)		*/

wire addr_alu_H_add_sub;			/* (addr_alu_H_add_sub = 0) => (add Rn with either Nn or 1, depends on linear_N_or_1)	*/
									/* (addr_alu_H_add_sub = 1) => (subtract Rn with either Nn or 1 depends on linear_N_or_1))		*/

wire [`addrbus] addr_alu_H_out;		/* the register's new value after passing through the address alu */


/*------*/
/* Low  */
/*------*/

wire addr_alu_L_N_or_1;				/* (addr_alu_L_N_or_1 = 0) => (choose Nn)		*/
									/* (addr_alu_L_N_or_1 = 1) => (choose 1)		*/

wire addr_alu_L_add_sub;			/* (addr_alu_L_add_sub = 0) => (add Rn with either Nn or 1, depends on linear_N_or_1)	*/
									/* (addr_alu_L_add_sub = 1) => (subtract Rn with either Nn or 1 depends on linear_N_or_1))		*/

wire [`addrbus] addr_alu_L_out;		/* the register's new value after passing through the address alu */



/*--------------------------------------------------------------*/
/* R register-files		Control and buses 						*/
/*--------------------------------------------------------------*/

/*-------------------*/
/* 		High R		 */
/*-------------------*/

wire [`addrbus] Rin1_H;						/* input data bus into High R port1 */

wire [1:0] Rin1_H_select;							/* select the input into Rin1_H  -  coming from stage 2 */


wire Rin2_H_select;					/* select the input into Rin2_H  -  coming from stage 3 */


wire [`addrbus] Rout1_H;					/* output bus 1 from the High R */
wire [`addrbus] Rout2_H;					/* output bus 3 from the High R */


wire [`rfbus] Rraddr1_H;					/* the read address for High R port1 */

wire Rraddr1_H_select;						/* selects the input into Rraddr1_H. */



wire [`rfbus] Rraddr2_H;					/* the read address for High R port3 */


wire [`rfbus] Rwaddr1_H;					/* the write address for High R port1 */

wire [1:0] Rwaddr1_H_select;				/* selects the input into Rwaddr1_H. */


wire [`rfbus] Rwaddr2_H;					/* the write address for High R port2 */

wire Rwaddr2_H_select;						/* selects the input into Rwaddr2_H. */

wire [1:0] Rraddr2_H_select;
wire Rwrite1_H;								/* write enable into High R */
wire Rwrite1_H_2;							/* write enable into High R from stage 2 */

wire Rwrite2_H;								/* write enable into High R */
wire Rwrite2_H_3;							/* write enable into High R from stage 3 */


/*-------------------*/
/* 		Low R		 */
/*-------------------*/

wire [`addrbus] Rin1_L;						/* input bus into the Low  R */

wire [1:0] Rin1_L_select;							/* select the input into Rin1_L  -  coming from stage 2 */



wire Rin2_L_select;					/* select the input into Rin2_L  -  coming from stage 3 */


wire [`addrbus] Rout1_L;					/* output bus 1 from the Low R */
wire [`addrbus] Rout2_L;					/* output bus 3 from the Low R */


wire [`rfbus] Rraddr1_L;					/* the read  address for Low  R port1 */

wire Rraddr1_L_select;						/* selects the input into Rraddr1_L. */


wire [`rfbus] Rraddr2_L;					/* the read  address for Low R port3 */


wire [`rfbus] Rwaddr1_L;					/* the write address for Low R port1 */

wire [1:0] Rwaddr1_L_select;				/* selects the input into Rwaddr1_L. */


wire [`rfbus] Rwaddr2_L;					/* the write address for Ligh R port2 */

wire Rwaddr2_L_select;						/* selects the input into Rwaddr2_L. */

wire [1:0] Rraddr2_L_select;
wire Rwrite1_L;								/* write enable into Low  R */
wire Rwrite1_L_2;							/* write enable into Low  R from stage 2 */

wire Rwrite2_L;								/* write enable into Low  R */
wire Rwrite2_L_3;							/* write enable into Low  R from stage 3 */


/*--------------------------------------------------------------*/
/* N wireister-files		Control and buses 						*/
/*--------------------------------------------------------------*/

/*----------------------*/
/* 		High N 			*/
/*----------------------*/

wire [`addrbus] Nin1_H;						/* input bus into the High N */


wire Nin2_H_select;					/* select the input into Nin2_H  -  coming from stage 3 */


wire [`addrbus] Nout1_H;					/* output bus 1 from the High N */
wire [`addrbus] Nout2_H;					/* output bus 3 from the High N */


wire [`rfbus] Nraddr1_H;					/* the read address for High N port1 */

wire Nraddr1_H_select;						/* selects the input into Nraddr1_H. */



wire [`rfbus] Nraddr2_H;					/* the read address for High N port3 */


wire [`rfbus] Nwaddr1_H;					/* the write address for High N port1 */


wire [`rfbus] Nwaddr2_H;					/* the write address for High N port2 */

wire Nwaddr2_H_select;						/* selects the input into Nwaddr2_H. */


wire [1:0] Nraddr2_H_select;

wire Nwrite1_H;								/* write enable into High N */
wire Nwrite1_H_2;							/* write enable into High N from stage 2 */

wire Nwrite2_H;								/* write enable into High N */
wire Nwrite2_H_3;							/* write enable into High N from stage 3 */


/*----------------------*/
/* 		Low N 			*/
/*----------------------*/

wire [`addrbus] Nin1_L;						/* input bus into the Ligh N */


wire Nin2_L_select;					/* select the input into Nin2_L  -  coming from stage 3 */


wire [`addrbus] Nout1_L;					/* output bus 1 from the Ligh N */
wire [`addrbus] Nout2_L;					/* output bus 3 from the Ligh N */



wire [`rfbus] Nraddr1_L;					/* the read address for Ligh N port1 */

wire Nraddr1_L_select;						/* selects the input into Nraddr1_L. */



wire [`rfbus] Nraddr2_L;					/* the read address for Ligh N port3 */



wire [`rfbus] Nwaddr1_L;					/* the write address for Ligh N port1 */



wire [`rfbus] Nwaddr2_L;					/* the write address for Ligh N port2 */

wire Nwaddr2_L_select;						/* selects the input into Nwaddr2_L. */

wire [1:0] Nraddr2_L_select;
wire Nwrite1_L;								/* write enable into Ligh N */
wire Nwrite1_L_2;							/* write enable into Ligh N from stage 2 */

wire Nwrite2_L;								/* write enable into Ligh N */
wire Nwrite2_L_3;							/* write enable into Ligh N from stage 3 */


/*--------------------------------------------------------------*/
/* M wireister-files		Control and buses						*/
/*--------------------------------------------------------------*/

/*----------------------*/
/* 		High M 			*/
/*----------------------*/

wire [`addrbus] Min1_H;						/* input bus into the High M */

wire Min2_H_select;					/* select the input into Min2_H  -  coming from stage 3 */


wire [`addrbus] Mout1_H;					/* output bus 1 from the High M */
wire [`addrbus] Mout2_H;					/* output bus 3 from the High M */



wire [`rfbus] Mraddr1_H;					/* the read address for High M port1 */

wire Mraddr1_H_select;						/* selects the input into Mraddr1_H. */



wire [`rfbus] Mraddr2_H;					/* the read address for High M port3 */


wire [`rfbus] Mwaddr1_H;					/* the write address for High M port1 */


wire [`rfbus] Mwaddr2_H;					/* the write address for High M port2 */

wire Mwaddr2_H_select;						/* selects the input into Mwaddr2_H. */

wire [1:0] Mraddr2_H_select;

wire Mwrite1_H;								/* write enable into High M */
wire Mwrite1_H_2;							/* write enable into High M from stage 2 */

wire Mwrite2_H;								/* write enable into High M */
wire Mwrite2_H_3;							/* write enable into High M from stage 3 */

 
/*----------------------*/
/* 		Low M 			*/
/*----------------------*/

wire [`addrbus] Min1_L;						/* input bus into the Ligh M */


wire Min2_L_select;					/* select the input into Min2_L  -  coming from stage 3 */


wire [`addrbus] Mout1_L;					/* output bus 1 from the Ligh M */
wire [`addrbus] Mout2_L;					/* output bus 3 from the Ligh M */



wire [`rfbus] Mraddr1_L;					/* the read address for Ligh M port1 */

wire Mraddr1_L_select;						/* selects the input into Mraddr1_L. */



wire [`rfbus] Mraddr2_L;					/* the read address for Ligh M port3 */


wire [`rfbus] Mwaddr1_L;					/* the write address for Ligh M port1 */


wire [`rfbus] Mwaddr2_L;					/* the write address for Ligh M port2 */

wire Mwaddr2_L_select;						/* selects the input into Mwaddr2_L. */
wire [1:0] Mraddr2_L_select;

wire Mwrite1_L;								/* write enable into Ligh M */
wire Mwrite1_L_2;							/* write enable into Ligh M from stage 2 */

wire Mwrite2_L;								/* write enable into Ligh M */
wire Mwrite2_L_3;							/* write enable into Ligh M from stage 3 */




wire regin2_H_select;	/* selecting the inputs for the 2nd readport */
wire regin2_L_select;

wire [1:0] regset_H_select;	/* selecting which register set to use */
wire [1:0] regset_L_select;

wire [15:0] regin2_L;	/* data into the 2nd write port of the register set */
wire [15:0] regin2_H;

/* the first two signals are unbuffered from agu_control */   
wire addr_aluL_ena_unbuf;
wire addr_aluH_ena_unbuf;
wire addr_aluL_ena;
wire addr_aluH_ena;

/* wires for outputs from guard latches */

   reg [15:0] aluinL_R;
   reg [15:0] aluinL_M;
   reg [15:0] aluinL_N;
   reg [15:0] aluinH_R;
   reg [15:0] aluinH_M;
   reg [15:0] aluinH_N;








   
/* wires for adding guard latches to the adders in address ALUs */
wire wrap_once_L_ena;
wire wrap_once_H_ena;

wire Clk_0_rfm;
wire Clk_1_rfn;
wire Clk_2_rfr;
wire Clk_3_ctrl;
wire Clk_4_ctrl;
wire Clk_5_misc;   
wire Clk_6_misc;   

// epoch set_attribute net Clk_0_rfm  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_0_rfm  : CLOCKGROUP = 1
// epoch set_attribute net Clk_1_rfn  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_1_rfn  : CLOCKGROUP = 1
// epoch set_attribute net Clk_2_rfr  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_2_rfr  : CLOCKGROUP = 1
// epoch set_attribute net Clk_3_ctrl : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_3_ctrl : CLOCKGROUP = 1
// epoch set_attribute net Clk_4_ctrl : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_4_ctrl : CLOCKGROUP = 1
// epoch set_attribute net Clk_5_misc : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_5_misc : CLOCKGROUP = 1
// epoch set_attribute net Clk_6_misc : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_6_misc : CLOCKGROUP = 1

wire reset_inv;


   assign reset_inv = ~reset;




/* wire for scan signals from control logic to registers */
wire control_scanout;

/*==============================================================================*/
/*============		Instantiated Modules	====================================*/
/*==============================================================================*/

assign Nwrite1_L = 1'b0;
assign Nwrite1_H = 1'b0;
assign Mwrite1_L = 1'b0;
assign Mwrite1_H = 1'b0;


/* agu control logic for stage 2 (Decode) */
agu_control_wrapper agu_control(
	.Clk_0 (Clk_3_ctrl),
	.Clk_1 (Clk_4_ctrl),
	.reset (reset),
	.pdb2 (PDB),
	.rep (REPEAT),
	.alt_pdb (pdb2),

	.scan_in(scanin),	
	.test(test),
	.scan_out(control_scanout),

	.xab_select (xab_select),
	.yab_select (yab_select),
	.PAB_2_select (PAB_2_select),
	.xwrite (xwrite),
	.xread (xread),
	.ywrite (ywrite),
	.yread (yread),
	.PABwrite_2 (PABwrite_2),
	.gdb_out_select (gdb_out_select),
	.gdb_write (gdb_write),
	.addr_alu_H_N_or_1 (addr_alu_H_N_or_1),
	.addr_alu_H_add_sub (addr_alu_H_add_sub),
	.addr_alu_L_N_or_1 (addr_alu_L_N_or_1),
	.addr_alu_L_add_sub (addr_alu_L_add_sub),
	.Rin1_H_select (Rin1_H_select),
	.Rraddr1_H_select (Rraddr1_H_select),
	.Rwaddr1_H_select (Rwaddr1_H_select),
	.Rwrite1_H_2 (Rwrite1_H_2),
	.Rin1_L_select (Rin1_L_select),
	.Rraddr1_L_select (Rraddr1_L_select),
	.Rwaddr1_L_select (Rwaddr1_L_select),
	.Rwrite1_L_2 (Rwrite1_L_2),
	.Nraddr1_H_select (Nraddr1_H_select),
	.Nraddr1_L_select (Nraddr1_L_select),
	.Mraddr1_H_select (Mraddr1_H_select),
	.Mraddr1_L_select (Mraddr1_L_select),

	.Rraddr2_L_select (Rraddr2_L_select),
	.Rraddr2_H_select (Rraddr2_H_select),
	.Nraddr2_L_select (Nraddr2_L_select),
	.Nraddr2_H_select (Nraddr2_H_select),
	.Mraddr2_L_select (Mraddr2_L_select),
	.Mraddr2_H_select (Mraddr2_H_select),
	.Rin2_H_select (Rin2_H_select),
	.Rwaddr2_H_select (Rwaddr2_H_select),
	.Rwrite2_H_3 (Rwrite2_H_3),
	.Rin2_L_select (Rin2_L_select),
	.Rwaddr2_L_select (Rwaddr2_L_select),
	.Rwrite2_L_3 (Rwrite2_L_3),
	.Nin2_H_select (Nin2_H_select),
	.Nwaddr2_H_select (Nwaddr2_H_select),
	.Nwrite2_H_3 (Nwrite2_H_3),
	.Nin2_L_select (Nin2_L_select),
	.Nwaddr2_L_select (Nwaddr2_L_select),
	.Nwrite2_L_3 (Nwrite2_L_3),
	.Min2_H_select (Min2_H_select),
	.Mwaddr2_H_select (Mwaddr2_H_select),
	.Mwrite2_H_3 (Mwrite2_H_3),
	.Min2_L_select (Min2_L_select),
	.Mwaddr2_L_select (Mwaddr2_L_select),
	.Mwrite2_L_3 (Mwrite2_L_3),

	.wrap_once_L_ena(wrap_once_L_ena),
	.wrap_once_H_ena(wrap_once_H_ena),

	.addr_aluL_ena(addr_aluL_ena_unbuf),
	.addr_aluH_ena(addr_aluH_ena_unbuf),
        .immediate3(immediate)
	);


/*==============================================================================*/
/*============		Instantiated Modules	====================================*/
/*==============================================================================*/


/*----------------------------------------------------------------------*/
/* PAB_2: there are 4 possible data inputs into PAB_2			*/
/*----------------------------------------------------------------------*/


   DW01_mux_any #(64, 2, 16) PAB_2_selector
       ({addr_alu_H_out, addr_alu_L_out, Rout1_H, Rout1_L}, PAB_2_select, PAB_2);  













/*----------------------------------------------------------------------*/
/* xab: there are 5 possible data inputs into xab						*/
/*----------------------------------------------------------------------*/


   DW01_mux_any #(80, 3, 16) xab_selector
       ({{10'h000, pdb2[13:8]}, addr_alu_H_out, addr_alu_L_out, Rout1_H, Rout1_L}
	, xab_select, xab); 












/*----------------------------------------------------------------------*/
/* yab: there are 5 possible data inputs into yab						*/
/*----------------------------------------------------------------------*/


   DW01_mux_any #(80, 3, 16) yab_selector
       ({{10'h000, pdb2[13:8]}, addr_alu_H_out, addr_alu_L_out, Rout1_H, Rout1_L}
	, yab_select, yab); 












/*----------------------------------------------------------------------*/
/* gdb_out: there are 8 possible data inputs into gdb_out				*/
/*----------------------------------------------------------------------*/


   DW01_mux_any #(128, 3, 16) gdb_out_selector
       ({Mout2_H, Mout2_L, Nout2_H, Nout2_L, Rout2_H, Rout2_L, Rin1_H, Rin1_L}
	, gdb_out_select, gdb_out); 















/*-------------------------------------------------------*/
/* 				address ALUs							 */
/*-------------------------------------------------------*/


/*** Guard latches to the address ALUs, 3 inputs to each of the 2 ALUs ***/
/*** Note: additional guard latches which were used to shut off
           part of the address ALUs are disabled since they lie
           in the critical paths of the AGU. Need to include the module
           addr_modifier if those guard latches are to be put back in the 
           design.                                            ***/

   always @(addr_aluL_ena_unbuf or addr_aluH_ena_unbuf 
            or Rout1_L or Mout1_L or Nout1_L
            or Rout1_H or Mout1_H or Nout1_H)
    begin
       if (addr_aluL_ena_unbuf)
	begin
	   aluinL_R <= Rout1_L;
	   aluinL_M <= Mout1_L;
	   aluinL_N <= Nout1_L;
	end
       if (addr_aluH_ena_unbuf)
	begin
	   aluinH_R <= Rout1_H;
	   aluinH_M <= Mout1_H;
	   aluinH_N <= Nout1_H;
	end // if (addr_aluH_ena)
    end // always @ (addr_aluL_ena or addr_aluH_ena)
   


















   
/*----------------------------------------------*/
/* address ALU for the High register-file banks */
/*----------------------------------------------*/

addr_alu #("alu_H") addr_alu_H(
	.Rn (aluinH_R),
	.Nn (aluinH_N),
	.Mn (aluinH_M),
	.N_or_1 (addr_alu_H_N_or_1),
	.add_sub (addr_alu_H_add_sub),
	.wrap_once_ena(wrap_once_H_ena),
	.register_new_value (addr_alu_H_out)
	);


/*---------------------------------------------*/
/* address ALU for the LOW register-file banks */
/*---------------------------------------------*/
addr_alu #("alu_L") addr_alu_L(
	.Rn (aluinL_R),
	.Nn (aluinL_N),
	.Mn (aluinL_M),
	.N_or_1 (addr_alu_L_N_or_1),
	.add_sub (addr_alu_L_add_sub),
	.wrap_once_ena(wrap_once_L_ena),
	.register_new_value (addr_alu_L_out)
	);




/*------------------------------------------------------*/
/* 				register files 							*/
/*------------------------------------------------------*/

/* For high bank registers */

   DW01_mux_any #(32, 1, 16) regin2_H_selector
       ({{8'h00, pdb3[15:8]}, GDB[`addrbus]}, regin2_H_select, regin2_H);  








  

   DW01_mux_any #(3, 2, 1) regset_H_selector
       ({Nin2_H_select, Min2_H_select, Rin2_H_select}, 
	regset_H_select, regin2_H_select);











regset_selector_c regset_H_selector_c(
	.Mwrite2 (Mwrite2_H),
	.Nwrite2 (Nwrite2_H),
	.Rwrite2 (Rwrite2_H),
	.regset_select (regset_H_select)
	);

/* For low bank registers */

   DW01_mux_any #(32, 1, 16) regin2_L_selector
       ({{8'h00, pdb3[15:8]}, GDB[`addrbus]}, regin2_L_select, regin2_L);  








   

   DW01_mux_any #(3, 2, 1) regset_L_selector
       ({Nin2_L_select, Min2_L_select, Rin2_L_select}, 
	regset_L_select, regin2_L_select);










   
regset_selector_c regset_L_selector_c(
	.Mwrite2 (Mwrite2_L),
	.Nwrite2 (Nwrite2_L),
	.Rwrite2 (Rwrite2_L),
	.regset_select (regset_L_select)
	);


/*----------------------*/
/*  R register files:	*/
/*	R_H	:	R7-R4		*/
/*	R_L	:	R0-R3		*/
/*----------------------*/

/*------------------------------*/
/*				High			*/
/*------------------------------*/

rfr #("alu_H")  R_H(
	.reset (reset),
	.in1 (Rin1_H),
	.in2 (regin2_H),
	.raddr1 (Rraddr1_H),
	.raddr2 (Rraddr2_H),
	.waddr1 (Rwaddr1_H),
	.waddr2 (Rwaddr2_H),
	.write1 (Rwrite1_H),
	.write2 (Rwrite2_H),
	.out1 (Rout1_H),
	.out2 (Rout2_H),
	.Clk (Clk_2_rfr)
	);



/* Rraddr1_H: there are 2 possible read addresses into Rraddr1_H */

   DW01_mux_any #(4, 1, 2) Rraddr1_H_selector
       ({pdb2[14:13], pdb2[9:8]}, Rraddr1_H_select, Rraddr1_H);  











   DW01_mux_any #(6, 2, 2) Rraddr2_H_selector
       ({pdb3[17:16], pdb3[14:13], pdb3[9:8]}, Rraddr2_H_select, Rraddr2_H);  











/* It is used in stage 2 for MOVE (X: or Y:). */ 


/* Rin1_H: there are 3 possible data inputs into Rin1_H */

   DW01_mux_any #(48, 2, 16) Rin1_H_selector
       ({addr_alu_L_out, GDB[`addrbus], addr_alu_H_out}, Rin1_H_select, Rin1_H);  












/* Rwaddr1_H: there are 3 possible write addresses into Rwaddr1_H */

   DW01_mux_any #(8, 2, 2) Rwaddr1_H_selector
    ({pdb2[1:0], pdb2[17:16], pdb2[14:13], pdb2[9:8]}, Rwaddr1_H_select, Rwaddr1_H);  
















/* Rwaddr2_H: there are 2 possible write addresses into Rwaddr2_H */

   DW01_mux_any #(4, 1, 2) Rwaddr2_H_selector
    ({pdb3[17:16], pdb3[9:8]}, Rwaddr2_H_select, Rwaddr2_H);  










/*--------------*/
/* write enable */
/*--------------*/

assign Rwrite2_H = Rwrite2_H_3;


/* Rwrite1_H */

write_enable_selector Rwrite1_H_selector(
	.write1_in (Rwrite1_H_2),
	.write2 (Rwrite2_H),
	.waddr1 (Rwaddr1_H),
	.waddr2 (Rwaddr2_H),
	.write1_out (Rwrite1_H)
	);
	
	

/*------------------------------*/
/*				Low				*/
/*------------------------------*/

rfr  #("alu_L") R_L(
	.reset (reset),
	.in1 (Rin1_L),
	.in2 (regin2_L),
	.raddr1 (Rraddr1_L),
	.raddr2 (Rraddr2_L),
	.waddr1 (Rwaddr1_L),
	.waddr2 (Rwaddr2_L),
	.write1 (Rwrite1_L),
	.write2 (Rwrite2_L),
	.out1 (Rout1_L),
	.out2 (Rout2_L),
	.Clk (Clk_2_rfr)
	);


/* Rraddr1_L: there are 2 possible read addresses into Rraddr1_L */

   DW01_mux_any #(4, 1, 2) Rraddr1_L_selector
    ({pdb2[14:13], pdb2[9:8]}, Rraddr1_L_select, Rraddr1_L);  











   DW01_mux_any #(6, 2, 2) Rraddr2_L_selector
    ({pdb3[17:16], pdb3[14:13], pdb3[9:8]}, Rraddr2_L_select, Rraddr2_L);  










   
/* It is used in stage 2 for MOVE (X: or Y:). */ 

/* Rin1_L: there are 3 possible data inputs Rin1_L */


   DW01_mux_any #(48, 2, 16) Rin1_L_selector
    ({addr_alu_H_out, GDB[`addrbus], addr_alu_L_out}, Rin1_L_select, Rin1_L);  












/* Rwaddr1_L: there are 3 possible write addresses into Rwaddr1_L */

   DW01_mux_any #(8, 2, 2) Rwaddr1_L_selector
       ({pdb2[1:0], pdb2[17:16], pdb2[14:13], pdb2[9:8]}, Rwaddr1_L_select, Rwaddr1_L);  














/* Rwaddr2_L: there are 2 possible write addresses into Rwaddr2_L */

   DW01_mux_any #(4, 1, 2) Rwaddr2_L_selector
       ({pdb3[17:16], pdb3[9:8]}, Rwaddr2_L_select, Rwaddr2_L);  










/*--------------*/
/* write enable */
/*--------------*/

assign Rwrite2_L = Rwrite2_L_3;


/* Rwrite1_L */

write_enable_selector Rwrite1_L_selector(
	.write1_in (Rwrite1_L_2),
	.write2 (Rwrite2_L),
	.waddr1 (Rwaddr1_L),
	.waddr2 (Rwaddr2_L),
	.write1_out (Rwrite1_L)
	);



/*----------------------*/
/*  M register files:	*/
/*	M_H	:	M7-M4		*/
/*	M_L	:	M0-M3		*/
/*----------------------*/

/*------------------------------*/
/*				High			*/
/*------------------------------*/
 
rfm #("alu_H") M_H (
	.reset  (reset),
	.in1    (regin2_H),
	.raddr1 (Mraddr1_H),
	.raddr2 (Mraddr2_H),
	.waddr1 (Mwaddr2_H),
	.write1 (Mwrite2_H),
	.out1   (Mout1_H),	
	.out2   (Mout2_H),	
	.Clk    (Clk_0_rfm)
	);


/* Mraddr1_H: there are 2 possible read addresses into Mraddr1_H */

   DW01_mux_any #(4, 1, 2) Mraddr1_H_selector
       ({pdb2[14:13], pdb2[9:8]}, Mraddr1_H_select, Mraddr1_H);  










   DW01_mux_any #(6, 2, 2) Mraddr2_H_selector
       ({pdb3[1:0], pdb3[14:13], pdb3[9:8]}, Mraddr2_H_select, Mraddr2_H);  











assign Min1_H = GDB[`addrbus];		/* the only input is coming from stage 2. formerly known as Min1_H_2 */

assign Mwaddr1_H = pdb3[1:0];		/* the only input is coming from stage 2. formerly known as Mwaddr1_H_2 */



/* Mwaddr2_H: there are 2 possible write addresses into Mwaddr2_H */


   DW01_mux_any #(4, 1, 2) Mwaddr2_H_selector
       ({pdb3[1:0], pdb3[9:8]}, Mwaddr2_H_select, Mwaddr2_H);  










/*--------------*/
/* write enable */
/*--------------*/

assign Mwrite2_H = Mwrite2_H_3;


/*------------------------------*/
/*				Low				*/
/*------------------------------*/

rfm #("alu_L") M_L (
	.reset  (reset),
	.in1    (regin2_L),
	.raddr1 (Mraddr1_L),
	.raddr2 (Mraddr2_L),
	.waddr1 (Mwaddr2_L),
	.write1 (Mwrite2_L),
	.out1   (Mout1_L),	
	.out2   (Mout2_L),	
	.Clk    (Clk_0_rfm)
	);


/* Mraddr1_L: there are 2 possible read addresses into Mraddr1_L */

   DW01_mux_any #(4, 1, 2) Mraddr1_L_selector
       ({pdb2[14:13], pdb2[9:8]}, Mraddr1_L_select, Mraddr1_L);  











   DW01_mux_any #(6, 2, 2) Mraddr2_L_selector
    ({pdb3[1:0], pdb3[14:13], pdb3[9:8]}, Mraddr2_L_select, Mraddr2_L);  










   
assign Min1_L = GDB[`addrbus];		/* the only input is coming from stage 2. formerly known as Min1_L_2 */

assign Mwaddr1_L = pdb3[1:0];		/* the only input is coming from stage 2. formerly known as Mwaddr1_L_2 */



/* Mwaddr2_L: there are 2 possible write addresses into Mwaddr2_L */

   DW01_mux_any #(4, 1, 2) Mwaddr2_L_selector
    ({pdb3[1:0],  pdb3[9:8]}, Mwaddr2_L_select, Mwaddr2_L);  











/*--------------*/
/* write enable */
/*--------------*/

assign Mwrite2_L = Mwrite2_L_3;

/*----------------------*/
/*  N register files:	*/
/*	N_H	:	N7-N4		*/
/*	N_L	:	N3-N0		*/
/*----------------------*/

/*------------------------------*/
/*				High			*/
/*------------------------------*/

rfn #("alu_H")  N_H(
	.reset (reset),
	.in1 (regin2_H),
	.raddr1 (Nraddr1_H),
	.raddr2 (Nraddr2_H),
	.waddr1 (Nwaddr2_H),
	.write1 (Nwrite2_H),
	.out1 (Nout1_H),
	.out2 (Nout2_H),
	.Clk (Clk_1_rfn)
	);


/* Nraddr1_H: there are 2 possible read addresses into Nraddr1_H */

   DW01_mux_any #(4, 1, 2) Nraddr1_H_selector
    ({pdb2[14:13],  pdb2[9:8]}, Nraddr1_H_select, Nraddr1_H);  











   DW01_mux_any #(6, 2, 2) Nraddr2_H_selector
    ({pdb3[17:16], pdb3[14:13],  pdb3[9:8]}, Nraddr2_H_select, Nraddr2_H);  











assign Nin1_H = GDB[`addrbus];		/* the only input is coming from stage 2. formerly known as Nin1_H_2 */

assign Nwaddr1_H = pdb3[17:16];		/* the only input is coming from stage 2. formerly known as Nwaddr1_H_2 */




/* Nwaddr2_H: there are 2 possible write addresses into Nwaddr2_H */

   DW01_mux_any #(4, 1, 2) Nwaddr2_H_selector
    ({pdb3[17:16],  pdb3[9:8]}, Nwaddr2_H_select, Nwaddr2_H);  











/*--------------*/
/* write enable */
/*--------------*/

assign Nwrite2_H = Nwrite2_H_3;


/*------------------------------*/
/*				Low				*/
/*------------------------------*/


rfn #("alu_L") N_L(
	.reset (reset),
	.in1 (regin2_L),
	.raddr1 (Nraddr1_L),
	.raddr2 (Nraddr2_L),
	.waddr1 (Nwaddr2_L),
	.write1 (Nwrite2_L),
	.out1 (Nout1_L),
	.out2 (Nout2_L),
	.Clk (Clk_1_rfn)
	);


/* Nraddr1_L: there are 2 possible read addresses into Nraddr1_L */

   DW01_mux_any #(4, 1, 2) Nraddr1_L_selector
    ({pdb2[14:13],  pdb2[9:8]}, Nraddr1_L_select, Nraddr1_L);  











assign Nin1_L = GDB[`addrbus];		/* the only input is coming from stage 2. formerly known as Nin1_L_2 */

assign Nwaddr1_L = pdb3[17:16];


   DW01_mux_any #(6, 2, 2) Nraddr2_L_selector
    ({pdb3[17:16], pdb3[14:13],  pdb3[9:8]}, Nraddr2_L_select, Nraddr2_L);  










   
/* Nwaddr2_L: there are 2 possible write addresses into Nwaddr2_L */

   DW01_mux_any #(4, 1, 2) Nwaddr2_L_selector
    ({pdb3[17:16],  pdb3[9:8]}, Nwaddr2_L_select, Nwaddr2_L);  










/*--------------*/
/* write enable */
/*--------------*/

assign Nwrite2_L = Nwrite2_L_3;



/*--------------------------------------------------*/
/*	AGU Tristates			            */
/*--------------------------------------------------*/

wire [`databus] GDB_out;
wire pab_tri_en, gdb_tri_en;
   
agu_tristate agu_tristate(	
	.gdb_out (gdb_out),
	.gdb_write (gdb_write),
	.PABwrite_2 (PABwrite_2),
	.test(test),
        .GDB_out(GDB_out),				
	.pab_tri_en (pab_tri_en),
	.gdb_tri_en (gdb_tri_en)
	);


   assign PAB = ~pab_tri_en ? PAB_reg_out : 16'bz;
   assign GDB = ~gdb_tri_en ? GDB_out : 24'bz;






/* =====================================================================*/
/* ===		Pipeline Register   -- Stage 2  		     ===*/
/* =====================================================================*/

   DW03_reg_s_pl #(24, 0) PDB2_DFF (PDB, Clk_5_misc, reset_inv, ~REPEAT, pdb2); 




/* =====================================================================*/
/* ===		Pipeline Register   -- Stage 3 		     	     ===*/
/* =====================================================================*/

   DW03_reg_s_pl #(24, 0) PDB3_DFF (pdb2, Clk_5_misc, reset_inv, 1'b1, pdb3); 





wire XWRITE_reg_out;
wire YWRITE_reg_out;

/* =====================================================================*/
/* ===		XAB Registers		 		     	     ===*/
/* =====================================================================*/

   DW03_reg_s_pl #(16, 0) XAB_DFF (xab, Clk_6_misc, reset_inv, 1'b1, XAB);
   DW03_reg_s_pl #(1, 0) XWRITE_DFF (xwrite, Clk_5_misc, reset_inv, 1'b1, XWRITE_reg_out);
   DW03_reg_s_pl #(1, 0) XREAD_DFF (xread, Clk_5_misc, reset_inv, 1'b1, XREAD);






/* =====================================================================*/
/* ===		YAB Registers		 		     	     ===*/
/* =====================================================================*/

   DW03_reg_s_pl #(16, 0) YAB_DFF (yab, Clk_6_misc, reset_inv, 1'b1, YAB);
   DW03_reg_s_pl #(1, 0) YWRITE_DFF (ywrite, Clk_6_misc, reset_inv, 1'b1, YWRITE_reg_out);
   DW03_reg_s_pl #(1, 0) YREAD_DFF (yread, Clk_6_misc, reset_inv, 1'b1, YREAD);





   
/* Make sure nothing is written to memory when doing scan test */
assign XWRITE = XWRITE_reg_out & !test;
assign YWRITE = YWRITE_reg_out & !test;

/* =====================================================================*/
/* ===		PAB Registers		 		     	     ===*/
/* =====================================================================*/

   DW03_reg_s_pl #(16, 0) PAB_DFF (PAB_2, Clk_6_misc, reset_inv, 1'b1, PAB_reg_out); 



assign scanout = PAB_reg_out[15];

   
endmodule	/* end module agu_structural */
