/*	File:	pcu_stage3.v	 									*/

/*	module name: pcu_stage3													*/

/*	Description: Control logic for the  Program Control Unit for the CMU DSP core.	*/


/*  Author:	Jonathan Tong 
                based on behavioral model by Nitzan Weinberg												*/



   `include "./verilog/globals.v"




module pcu_stage3 (
	reset,
	pdb3,
	PAB_in,
	GDB,
        PDB,
	CCR,
        MR,
        PC,
        SP,
        LF,
        DM,
        S1,
        S0,
        I1,
        I0,
        OMR,

        PCwrite_3,
        LCwrite_repeat,
        LCwrite_do,
        SSLwrite_3,
        SSHwrite_3,
        SPwrite_3,
        LFwrite_3,
        DMwrite,
        S1write,
        S0write,
        I1write,
        I0write,
        OMRwrite,
        PC_in_3,
        LF_in_3,
        LC_repeat,
        DM_in,
        S1_in,
        S0_in,
        I1_in,
        I0_in,
        OMR_in,
        SSH_in_3,
        SSL_in_3,
        SP_in_3,
        LC_do
        );


/*==================================================================================*/
/*============						I/O direction						============*/
/*==================================================================================*/

input reset;
input [`databus] pdb3;
input [`addrbus] PAB_in;
input [`addrbus] GDB;
input [`databus] PDB;

input [7:0] CCR;
input [7:0] MR;
input [`addrbus] PC;
input [`addrbus] SP;
input LF;
input DM;
input S1;
input S0;
input I1;
input I0;
input [23:0] OMR;

output PCwrite_3;
output LCwrite_repeat;
output LCwrite_do;
output SSLwrite_3;
output SSHwrite_3;
output SPwrite_3;
output LFwrite_3;
output DMwrite;
output S1write;
output S0write;
output I1write;
output I0write;
output OMRwrite;
output [`addrbus] PC_in_3;
output LF_in_3;
output [`addrbus] LC_repeat;
output DM_in;
output S1_in;
output S0_in;
output I1_in;
output I0_in;
output [23:0] OMR_in;
output [`addrbus] SSH_in_3;
output [`addrbus] SSL_in_3;
output [`addrbus] SP_in_3;
output [`addrbus] LC_do;


/*==================================================================================*/
/*============							I/O type						============*/
/*==================================================================================*/

reg PCwrite_3;
reg LCwrite_repeat;
reg LCwrite_do;
reg SSLwrite_3;
reg SSHwrite_3;
reg SPwrite_3;
reg LFwrite_3;
reg DMwrite;
reg S1write;
reg S0write;
reg I1write;
reg I0write;
reg OMRwrite;
reg [`addrbus] PC_in_3;
reg LF_in_3;
reg [`addrbus] LC_repeat;
reg DM_in;
reg S1_in;
reg S0_in;
reg I1_in;
reg I0_in;
reg [23:0] OMR_in;
reg [`addrbus] SSH_in_3;
reg [`addrbus] SSL_in_3;
reg [`addrbus] SP_in_3;
reg [`addrbus] LC_do;


/*==================================================================================*/
/*===========						Internal Nets						============*/
/*==================================================================================*/




/*--------------------------------------------------------------------------------------*/
/*																						*/
/* 		Logic Section																	*/
/*																						*/
/*--------------------------------------------------------------------------------------*/

always @(	pdb3 or 
			reset or 
			PAB_in or
			GDB or
                      //  PDB or  /* latches will be infered but that's ok */
			CCR or
                        OMR or
			MR or
			PC or
			SP or
                        LF or
                        DM or
                        S0 or
                        S1 or
                        I0 or
                        I1 or
		PDB
			)
	begin

		/*---------------------*/
		/* initialize controls */
		/*---------------------*/

		PCwrite_3 = `false;
		
		LCwrite_repeat = `false;

		LCwrite_do = `false;

		SSLwrite_3 = `false;		
		SSHwrite_3 = `false;		

		SPwrite_3 = `false;

		/* initialize Mode Register bits, and their write enable */

		LFwrite_3 = `false;
		DMwrite = `false;
		S1write = `false;
		S0write = `false;
		I1write = `false;
		I0write = `false;

		OMRwrite = `false;

		/* initialize the data , for synthesis */

		DM_in = 1'bx;
		I0_in = 1'bx;
		I1_in = 1'bx;
	  
	        LC_do = 16'hx;
      		LC_repeat = 16'hx;
		LF_in_3 = 1'bx;
	        OMR_in = 8'hx;
	        PC_in_3 = 16'hx;
	        S0_in = 1'bx;
        	S1_in = 1'bx;
	        SP_in_3 = 16'bx;
	        SSH_in_3 = 16'bx;
	        SSL_in_3 = 16'bx;

		/*---------------------------------*/
		/* Decode the incoming instruction */
		/*---------------------------------*/
		
		if ( {pdb3[23:14], pdb3[7:0]} == 18'b0000_1010_11_1000_0000 )
			begin	/*--------------------------------------*/
				/*	JMP Class II			*/
				/*					*/
				/* Jump with an effective address	*/
				/*--------------------------------------*/
					
				/* The effective address is generated by the AGU */
				/* Get the new address from the PAB_in and increment it by 1 in preparation for the next cycle */

				if (pdb3[13:8] == 6'b110000)
					begin
						PC_in_3 = PDB[15:0];				/* the absolute address is coming from the program memory */
						//PCwrite_3 = `true;
						PCwrite_3 = `false;
					end
				else
					begin
						PC_in_3 = PAB_in + 1;				/* generated by the AGU and increment it by one */
						PCwrite_3 = `true;
					end
				
			end		/* JMP Class II */
			



		else if ( {pdb3[23:14], pdb3[7:0]} == 18'b00000110_11_00100000 )
			begin	/*--------------------------------------*/
					/*				REP Class IV			*/
					/*										*/
					/* Repeat Next Instruction				*/
					/*--------------------------------------*/

				/* number of repetitions is coming on the GDB from either the AGU or the data ALU */
				
				if ( (pdb3[13:12] == 2'b00) || (pdb3[13:11] == 3'b010) || (pdb3[13:11] == 3'b011) || (pdb3[13:11] == 3'b100) )
					begin
						LCwrite_repeat = `true;		/* enable latching of the initial loop counter coming from memory on the GDB bus */
						LC_repeat = GDB;			/* 16 bit only */
					end
					
			end		/* REP Class IV */




		else if ( {pdb3[23:15], pdb3[7], pdb3[5:0]} == 16'b00000110_0_0_100000 )
			begin	/*--------------------------------------*/
					/*			REP Class I	or II			*/
					/*										*/
					/* Repeat Next Instruction				*/
					/*--------------------------------------*/
					
				LCwrite_repeat = `true;		/* enable latching of the initial loop counter coming from memory on the GDB bus */
				LC_repeat = GDB;			/* 16 bit only */
				
			end		/* REP Class I or II */




		else if ( {pdb3[23:16], pdb3[7:2]} == 14'b00000000_101110 )
			begin	/*--------------------------------------*/
					/*					ANDI				*/
					/*										*/
					/* And Immediate with Control Register	*/
					/*--------------------------------------*/
				case (pdb3[1:0])
					2'b00	:	/* MR */
								begin
									LF_in_3 = pdb3[15] && LF;
									LFwrite_3 = `true;
									DM_in = pdb3[14] && DM;
									DMwrite = `true;
									S1_in = pdb3[11] && S1;
									S1write = `true;
									S0_in = pdb3[10] && S0;
									S0write = `true;
									I1_in = pdb3[ 9] && I1;
									I1write = `true;
									I0_in = pdb3[ 8] && I0;
									I0write = `true;
								end

					2'b01	:	;	/* No Action - CCR are taken care of in the data alu */

					2'b10	:	/* OMR */
								begin
									OMR_in[7:0] = pdb3[15:8] & OMR[7:0];
									OMRwrite = `true;
								end

					default	:	;		/* No Action */

				endcase		/* pdb3[1:0] */

			end		/* ANDI */




		else if ( ({pdb3[23:14], pdb3[7:6]} == 12'b00001010_11_01) ||
				  ({pdb3[23:14], pdb3[7:5]} == 13'b00001011_11_011) )
				  
			begin	/*--------------------------------------*/
					/*		BCLR / BSET / BTST Class III	*/
					/*										*/
					/*	Bit Test and Clear/Set - Class III	*/
					/*--------------------------------------*/


				
			end		/* BCLR/BSET/BTST Class III */




		else if ( {pdb3[23:16], pdb3[7:4]} == 12'b00000110_1010 )
			begin	/*--------------------------------------*/
					/*			REP Class III				*/
					/*										*/
					/* Repeat Next Instruction				*/
					/*--------------------------------------*/
					
				/* Immediate data in the program instruction */

				LCwrite_repeat = `true;		/* enable latching of the initial loop counter coming from memory on the GDB bus */
				LC_repeat = {4'b0000, pdb3[3:0], pdb3[15:8]};	/* pay attention to the special concatenation !!! */
				
			end		/* REP Class III */




		else if ( {pdb3[23:16], pdb3[7:4]} == 12'b0000_0110_1000 || {pdb3[23:14], pdb3[7:0]} == 18'b0000_0110_11_0000_0000 )
			begin	        /*--------------------------------------*/
					/*			DO Class III				*/
					/*			DO Class IV				*/
					/* Start Hardware Loop					*/
					/*--------------------------------------*/
				
				
				SSH_in_3 = PC;				/* load the PC of the first instruction in the loop in SSH */
				SSHwrite_3 = `true;
				
				SSL_in_3 = {MR, CCR};		/* load the current SR = {MR, CCR} in the SSL */
				SSLwrite_3 = `true;

				SP_in_3 = SP[5:0] + 1'h1;		/* adjust the stack pointer */
				SPwrite_3 = `true;
				
				LF_in_3 = `true;				/* set loop flag */
				LFwrite_3 = `true;
				
				if ({pdb3[23:16], pdb3[7:4]} == 12'b0000_0110_1000)
			
				/* an immediate short repetition number is encoded in the program word */
					LC_do = {4'b0000, pdb3[3:0], pdb3[15:8]};		/* immediate short data */
				else
					LC_do = GDB[15:0];
				LCwrite_do = `true;

			end		/* DO Class III */
			



		else if ( ({pdb3[23:14], pdb3[7]}   == 11'b00001010_01_1) ||
				  ({pdb3[23:14], pdb3[7]}   == 11'b00001010_00_1) ||
				  ({pdb3[23:14], pdb3[7:6]} == 12'b00001010_11_00) )
			begin	/*----------------------------------------------*/
					/*		JCLR / JSET Class I, II or III			*/
					/*												*/
					/*	Jump if Bit Clear/Set - Class I, II or III	*/
					/*----------------------------------------------*/



			end		/* JCLR/JSET Class I, II or III */




		else if ( {pdb3[23:16], pdb3[7:5]} == 11'b00000101_101 )
			begin	/*----------------------------------*/
					/*		MOVEC Class IV				*/
					/*									*/
					/* No Action				 		*/
					/*----------------------------------*/
			end		/* MOVEC  Class IV */




		else if ( {pdb3[23:16], pdb3[7], pdb3[5]} == 10'b00000101_0_1 )
			begin	/*----------------------------------*/
					/*		MOVEC Class I and II		*/
					/*									*/
					/* No Action				 		*/
					/*----------------------------------*/
			end		/* MOVEC Class I and II */






	end		/* @(pdb3) */





endmodule	/* pcu_stage3 */
