/********************************************************************************
 *	File:	pcu.v
 ********************************************************************************
 *	module name: pcu
 *	Description: The Program Control Unit for the CMUDSP core.
 *      Author:	Nitzan Weinberg	, Jonathan Tong
 *
 * $Revision: 1.35 $
 * $Date: 1998/08/30 03:31:05 $
 *
 * Scan chain sequence:
 *
 * scan_in -> LF(1) -> DM(1) -> unbuf_S1(1) -> unbuf_S0(1) -> I1(1) -> I0(1)
 *            -> OMR(24) -> S(1) -> L(1) -> E(1) -> U(1) -> N(1) -> Z(1) ->
 *            V(1) -> C(1) -> PC(16) -> unbuf_REPEAT(1) -> LC(16) ->
 *            LC_temp(16) -> LA(16) -> SP(16) -> PDB2(24) -> PDB3(24) ->
 *            PABreg_out(16) -> PABwrite_out(1) -> scan_out
 *								
 *****************************************************************************/


/*==============================================================================*/




   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"



/********************************************************************************/
/*								                */
/*	module:	pcu_structural			                                */
/*										*/
/********************************************************************************/

module pcu_structural (
	reset,
	immediate,	       
	PDB,
	GDB,
	PAB,
	CCR_from_alu,
	Swrite,
	Lwrite,
	J,
	REPEAT,
	CCR,
	S1,
	S0,
	scan_in, test, scan_out,
	Clk_0,
	Clk_1
	);


/*==================================================================================*/
/*============						I/O direction						============*/
/*==================================================================================*/

input reset;
input immediate;   /* from ALU, ignore the current word on PDB, not an instrution */   
input [`databus] PDB;
input [`addrbus] GDB;
input [7:0] CCR_from_alu;
input Swrite;
input Lwrite;
input J;

input	scan_in, test;
output	scan_out;

inout [`addrbus] PAB;
output REPEAT;
output [7:0] CCR;
output S1;
output S0;

input Clk_0, Clk_1;


/*==================================================================================*/
/*============							I/O type						============*/
/*==================================================================================*/

wire reset;
wire [`databus] PDB;
wire [`addrbus] GDB;
wire [7:0] CCR;				/* condition code register from the data alu */
wire J;					/* jump indicated by the AGU or the data alu */
wire [`addrbus] PAB;		        /* program address bus for use with branch commands */

wire  REPEAT;				/* repeat last word instruction signal */

wire Clk_0, Clk_1;

// epoch set_attribute net Clk_0  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_0  : CLOCKGROUP = 1
// epoch set_attribute net Clk_1  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_1  : CLOCKGROUP = 1

/*==================================================================================*/
/*===========						Internal Nets						============*/
/*==================================================================================*/

wire [`databus] pdb2;		/* program instruction that is processed during stage 2 (Decode) */
wire [`databus] pdb3;		/* program instruction that is processed during stage 3 (Execute) */


wire PABwrite;				/* write enable to the PAB */


/*----------*/
/* PC		*/
/*----------*/

wire [`addrbus] PC;			/* program counter REGISTER 									*/
wire [`addrbus] PC_in;		/* the input to the program counter register					*/
wire [`addrbus] PC_in_2;		/* the new value for the program counter from stage 2			*/
wire [`addrbus] PC_in_3;		/* the new value for the program counter from stage 3			*/
wire PCwrite;				/* PC register write enable										*/
wire PCwrite_2;				/* program counter write enable for value coming from stage 2	*/
wire PCwrite_3;				/* program counter write enable for value coming from stage 3	*/


/*--------------*/
/* System Stack */
/*--------------*/

wire [`addrbus] SP;				/* Stack pointer REGISTER 					*/
wire [`addrbus] SP_in;			/* the input for the stack pointer register	*/
wire [`addrbus] SP_in_2;			/* the new value for the SP from stage 2	*/
wire [`addrbus] SP_in_3;			/* the new value for the SP from stage 3	*/
wire SPwrite;					/* stack pointer write enable				*/
wire SPwrite_2;					/* SP write enable from stage 2				*/
wire SPwrite_3;					/* SP write enable from stage 3				*/

wire [`addrbus] SSL_in;			/* System Stack Low input					*/
wire [`addrbus] SSL_in_2;		/* System Stack Low input from stage 2		*/
wire [`addrbus] SSL_in_3;		/* System Stack Low input from stage 3		*/
wire SSLwrite;					/* SSL write enable							*/
wire SSLwrite_2;					/* SSL write enable from stage 2			*/
wire SSLwrite_3;					/* SSL write enable from stage 3			*/


wire [`addrbus] SSH_in;			/* System Stack High input					*/
wire [`addrbus] SSH_in_2;		/* System Stack High input from stage 2		*/
wire [`addrbus] SSH_in_3;		/* System Stack High input from stage 3		*/
wire SSHwrite;					/* SSH write enable							*/
wire SSHwrite_2;					/* SSH write enable from stage 2			*/
wire SSHwrite_3;					/* SSH write enable from stage 3			*/


/*----------*/
/* LC		*/
/*----------*/

wire [`addrbus] LC;				/* Loop counter REGISTER										*/
wire [`addrbus] LC_in;			/* the input to the loop counter register						*/
wire [`addrbus] LC_repeat;		/* the initial number of repetitions for a REP inst				*/
wire [`addrbus] LC_do;			/* the initial number of iterations for a DO loop, from stage 3	*/

wire LCwrite;					/* the write enable for the loop counter register				*/ 
wire LCwrite_repeat;				/* the write enable for the loop counter's initial value		*/ 
wire LCwrite_do;					/* the write enable for the loop counter from stage 3 (DO)		*/ 



/*-----------*/
/* LC_temp   */
/*-----------*/

wire LC_temp_write;
wire [`addrbus] LC_temp_in;
wire [`addrbus] LC_temp;

/*----------*/
/* LA		*/
/*----------*/

wire [`addrbus] LA;				/* Loop address REGISTER					*/
wire [`addrbus] LA_in;			/* the new input to the loop address		*/
wire LAwrite;					/* LA register write enable					*/ 
wire LAwrite_2;					/* loop address write enable from stage 2	*/ 


/*--------*/
/* repeat */
/*--------*/

wire REPEATset;				/* set the repeat signal (flip-flop) */
wire REPEATreset;			/* reset the repeat signal (flip-flop) */


/*----------------------*/
/* MR - mode register	*/
/*----------------------*/

wire [7:0] MR;				/* mode register */


/*---------------------------------------------------*/
/* Status Register (SR) - Mode Register (MR) portion */
/*---------------------------------------------------*/

wire LF;						/* bit 15 - Loop Flag */
wire LF_in;								/* The new LF bit before latching */
wire LF_in_3;							/* A new LF bit coming from stage 3 (DO instruction begins) */
wire LFwrite;							/* write enable */
wire LFwrite_3;							/* write enable from stage 3 to start a DO loop */
wire LF_reset;							/* reset LF at the end of a DO instruction */
wire LF_restore;							/* restore LF at the end of the last DO loop from the SSH */

wire DM;						/* bit 14 - Double Precision Multiply Mode */
wire DM_in;								/* The new DM bit before latching */
wire DMwrite;							/* write enable */

wire S1;						/* bit 11 - scaling mode */
wire S1_in;								/* The new S1 bit before latching */
wire S1write;							/* write enable */

wire S0;						/* bit 10 - scaling mode */
wire S0_in;								/* The new S0 bit before latching */
wire S0write;							/* write enable */

wire I1;						/* bit  9 - Interrupt Mask */
wire I1_in;								/* The new I1 bit before latching */
wire I1write;							/* write enable */

wire I0;						/* bit  8 - Interrupt Mask */
wire I0_in;								/* The new I0 bit before latching */
wire I0write;							/* write enable */


/*----------------------------------*/
/* Operating Mode Register (OMR)	*/
/*----------------------------------*/

wire [23:0] OMR;				/* Latched OMR */
wire [23:0] OMR_in;			/* The new OMR content before latching */

wire OMRwrite;				/* write enable */


/*--------------------------------------------------------------*/
/* Status Register (SR) - Condition Code Register (CCR) portion */
/*--------------------------------------------------------------*/

/* All are TRUE condition code bits - `true when the condition is met	*/

/* latched	bits */
/*---------------*/
wire S;						/* bit 7 - Scaling		*/

wire L;						/* bit 6 - Limit		*/


/* NOT latched	bits	*/
/*----------------------*/

/* The following bits are calculated from the result at the end of a data ALU operation.	*/

wire E;						/* bit 5 - Extension	*/

wire U;						/* bit 4 - Unnormalized	*/

wire N;						/* bit 3 - Negative		*/

wire Z;						/* bit 2 - Zero			*/

wire V;						/* bit 1 - Overflow		*/

wire C;						/* bit 0 - Carry		*/





wire [`addrbus] LF_out;
wire [`addrbus] PC_out;
wire [`addrbus] LC_out;
wire [`addrbus] LA_out;

wire [3:0] SSa;
wire [3:0] SSb;

wire SSawe;
wire SSawr;
wire SSbwr;

reg  REPEAT_in;
wire [15:0] PAB_reg_out;

/* =============================== */
/* intenal wires before buffering  */
/* =============================== */
wire unbuf_REPEAT;
wire unbuf_S0;
wire unbuf_S1;

/* =============================== */
/* intenal wires before delay buffers  */
/* =============================== */
wire [31:0] nodelay_SSadin;
wire [3:0] nodelay_SSa;

/*** tri-state buffer enable signal for PAB ***/
wire pab_tri_en;
   

/*==================================================================================*/
/*===========					Module Instantiation					============*/
/*==================================================================================*/

pcu_stage2 pcu_stage2_control (
	reset,
	pdb2,
	PDB,
	REPEAT,
	PC,
	LA,
        LC,
        SP,
        PCwrite_2,
        REPEATset,
        SSLwrite_2,
        SSHwrite_2,
        SPwrite_2,
        LAwrite_2,
        PC_in_2,
        SSH_in_2,
        SSL_in_2,
        SP_in_2
	);

pcu_stage3 pcu_stage3_control (
	reset,
	pdb3,
	PAB,
	GDB,
        PDB,

	CCR,
        MR,
        PC,
        SP,
        LF,
        DM,
        S1,
        S0,
        I1,
        I0,
        OMR,

        PCwrite_3,
        LCwrite_repeat,
        LCwrite_do,
        SSLwrite_3,
        SSHwrite_3,
        SPwrite_3,
        LFwrite_3,
        DMwrite,
        S1write,
        S0write,
        I1write,
        I0write,
        OMRwrite,
        PC_in_3,
        LF_in_3,
        LC_repeat,
        DM_in,
        S1_in,
        S0_in,
        I1_in,
        I0_in,
        OMR_in,
        SSH_in_3,
        SSL_in_3,
        SP_in_3,
        LC_do
	);


pcu_control pcu_control (
	reset,
        REPEAT,
	LF,
        PC,
        LA,
        LC,
        SP,
        PCwrite_2,
        PCwrite_3,
        PC_in_2,
        PC_in_3,
	LF_out,
	PC_out,
	LC_out,
	LA_out,
        LF_in_3,
        LFwrite_3,
        LCwrite_repeat,
 	LC_repeat,
	LC_temp,
        LC_do,
	LCwrite_do,
	LAwrite_2,
        SP_in_2,
  	SP_in_3,
	SPwrite_2,
	SPwrite_3,
	SSL_in_2,
	SSL_in_3,
   	SSLwrite_2,
	SSLwrite_3,
  	SSH_in_2,
	SSH_in_3,
   	SSHwrite_2,
	SSHwrite_3,
        PDB,
	REPEATset,
	PABwrite_out,
	test,

        PCwrite,
	PC_in,
	LFwrite,
	LF_in,
	LCwrite,
	LC_in,
	LAwrite,
	LA_in,
	SPwrite,
	SP_in,
	SSLwrite,
	SSL_in,
	SSHwrite,
	SSH_in,
	REPEATreset,
        PABwrite,
	nodelay_SSa,
	SSb,
	LC_temp_write,
	LC_temp_in,
	pab_tri_en			 
	);

/*===========================================================================*/
/*===================  Instantiated Inverters ===============================*/
/*===========================================================================*/

wire reset_inv;
wire LFwrite_inv;
wire DMwrite_inv;
wire S1write_inv;
wire S0write_inv;
wire I1write_inv;
wire I0write_inv;
wire OMRwrite_inv;
wire Swrite_inv;
wire Lwrite_inv;
wire PCwrite_inv;
wire LCwrite_inv;
wire LAwrite_inv;
wire SPwrite_inv;
wire SSawe_inv;



  /*** if using DesignWare, can just negate the signals by using '~' ***/


















/*==============================================================================================*/
/*==============================================================================================*/
/*																								*/
/*	Processes								Processes							Processes		*/
/*																								*/
/*==============================================================================================*/
/*==============================================================================================*/


/*==============================================================================================*/
/*																								*/
/*	Mode Register (MR) Bits																		*/
/*																								*/
/*==============================================================================================*/

assign MR = {LF, DM, 1'b0, 1'b0, S1, S0, I1, I0};



   DW03_reg_s_pl #(1, 0) LF_DFF (LF_in, Clk_0, ~reset, LFwrite, LF);
   DW03_reg_s_pl #(1, 0) DM_DFF (DM_in, Clk_0, ~reset, DMwrite, DM);
   DW03_reg_s_pl #(1, 0) S1_DFF (S1_in, Clk_0, ~reset, S1write, S1);
   DW03_reg_s_pl #(1, 0) S0_DFF (S0_in, Clk_0, ~reset, S0write, S0);
   DW03_reg_s_pl #(1, 0) I1_DFF (I1_in, Clk_0, ~reset, I1write, I1);
   DW03_reg_s_pl #(1, 0) I0_DFF (I0_in, Clk_0, ~reset, I0write, I0);
   DW03_reg_s_pl #(24, 0) OMR_REGISTER (OMR_in, Clk_0, ~reset, OMRwrite, OMR);













/*==============================================================================================*/
/*																								*/
/*	Latched Status Register Bits																*/
/*																								*/
/*==============================================================================================*/

assign CCR = {S, L, E, U, N, Z, V, C};


   DW03_reg_s_pl #(1, 0) S_DFF (CCR_from_alu[7], Clk_0, ~reset, Swrite, S);
   DW03_reg_s_pl #(1, 0) L_DFF (CCR_from_alu[6], Clk_0, ~reset, Lwrite, L);
   DW03_reg_s_pl #(1, 0) E_DFF (CCR_from_alu[5], Clk_0, ~reset, 1'b1, E);
   DW03_reg_s_pl #(1, 0) U_DFF (CCR_from_alu[4], Clk_0, ~reset, 1'b1, U);
   DW03_reg_s_pl #(1, 0) N_DFF (CCR_from_alu[3], Clk_0, ~reset, 1'b1, N);
   DW03_reg_s_pl #(1, 0) Z_DFF (CCR_from_alu[2], Clk_0, ~reset, 1'b1, Z);
   DW03_reg_s_pl #(1, 0) V_DFF (CCR_from_alu[1], Clk_0, ~reset, 1'b1, V);
   DW03_reg_s_pl #(1, 0) C_DFF (CCR_from_alu[0], Clk_0, ~reset, 1'b1, C);













/*----------------------------------*/
/* PC register: latching the new PC */
/*----------------------------------*/

   DW03_reg_s_pl #(16, 0) PC_REGISTER (PC_in, Clk_0, ~reset, PCwrite, PC);



/*==============================================================================================*/
/*																								*/
/*	REPEAT signal																				*/
/*																								*/
/*==============================================================================================*/

/*** should be a sync set/reset flip-flop ***/

   DW03_reg_s_pl #(1, 0) REPEAT_REGISTER (REPEAT_in, Clk_0, ~reset, 1'b1, REPEAT);





   
always @(reset or REPEATreset or REPEATset or REPEAT)
	begin
		if (reset || REPEATreset)	
			REPEAT_in = `false;
		else if (REPEATset)		
			REPEAT_in = `true;
		else
			REPEAT_in = REPEAT;
	end




/*==============================================================================================*/
/*																								*/
/*	Loop Counter (LC)																			*/
/*																								*/
/*==============================================================================================*/

/*-------------*/
/* LC register */
/*-------------*/

   DW03_reg_s_pl #(16, 0) LC_REGISTER (LC_in, Clk_0, ~reset, LCwrite, LC); 



   
/*-------------*/
/* LC_temp register */
/*-------------*/

   DW03_reg_s_pl #(16, 0) LC_temp_REGISTER (LC_temp_in, Clk_0, ~reset, LC_temp_write, LC_temp);



   
/*-------------*/
/* LA register */
/*-------------*/

   DW03_reg_s_pl #(16, 0) LA_REGISTER (LA_in, Clk_0, ~reset, LAwrite, LA);




/*-------------*/
/* SP register */
/*-------------*/

   DW03_reg_s_pl #(16, 0) SP_REGISTER (SP_in, Clk_0, ~reset, SPwrite, SP);



/*-----------*/
/* SSL & SSH */
/*-----------*/

assign SSbwr = 1'b1;

/* Write enables of the register files are activated only during ... */
/* .. the second half of the clock. SP only changes right after the rising clock edge so ... */
/* ... no address changes when write enables are high */
assign SSawe = SSHwrite | SSLwrite;

assign SSawr = ~SSawe | Clk_1;

wire [31:0] SSadin;
wire [31:0] SSbdin;
wire [31:0] SSadout;
wire [31:0] SSbdout;

/*============ added to increase the hold time of SSadin and SSa ============*/

/*====== 1 ns delay should be enough============== */

   assign SSa = nodelay_SSa;
   DW03_reg_s_pl #(32, 0) SSadin_REGISTER (nodelay_SSadin, ~Clk_1, ~reset, 1'b1, SSadin);




   
// dff_c #(32,1,"SSadin") SSadin_REGISTER(~Clk, reset_inv, SSadin_D, SSadin); 
/*===========================================================================*/

assign nodelay_SSadin = {SSH_in,SSL_in};
assign SSbdin = 32'h0;


   DW_ram_2r_w_a_lat #(32, 16, 1) SS(.data_in(SSadin),
				     .rd1_addr(SSa),
				     .rd2_addr(SSb),
				     .wr_addr(SSa),
				     .wr_n(SSawr),
				     .rst_n(),
				     .cs_n(1'b0),
				     .data_rd1_out(SSadout),
				     .data_rd2_out(SSbdout));



   
assign PC_out = SSadout[31:16];
assign LF_out = SSadout[15:0];
assign LA_out = SSbdout[31:16];
assign LC_out = SSbdout[15:0];



/*==============================================================================================*/
/*==============================================================================================*/
/*																								*/
/*	Stage 2 (Decode)					Stage 2 (Decode)					Stage 2 (Decode)	*/
/*																								*/
/*==============================================================================================*/
/*==============================================================================================*/

wire [`databus] pdb2_in;	

assign pdb2_in = (immediate) ? `NOP : PDB; /* if PDB is not an instruction, put NOP into pdb2 to prevent any decoding */


   DW03_reg_s_pl #(24, 0) PDB2_DFF (pdb2_in, Clk_1, ~reset, ~REPEAT, pdb2);






/*==============================================================================================*/
/*==============================================================================================*/
/*													*/
/*	Stage 3 (Execute)						Stage 3 (Execute)				Stage 3 (Execute)	*/
/*																								*/
/*==============================================================================================*/
/*==============================================================================================*/


   DW03_reg_s_pl #(24, 0) PDB3_DFF (pdb2, Clk_1, ~reset, 1'b1, pdb3);




/* =====================================================================*/
/* ===		PAB Registers		 		     	     ===*/
/* =====================================================================*/


   DW03_reg_s_pl #(16, 0) PAB_DFF (PC_in, Clk_1, ~reset, 1'b1, PAB_reg_out);
   DW03_reg_s_pl #(1, 1) PABwrite_DFF (PABwrite, Clk_1, ~reset, 1'b1, PABwrite_out);







   assign PAB = pab_tri_en ? 16'bz : PAB_reg_out;   // pab_tri_en is active low




   
assign scan_out = PABwrite_out;

endmodule	/* pcu_structural */



