/*	File:	write_enable_selector.v	  					*/

/*	module name: write_enable_selector					*/

/*	Description:	A write enable selector for port 1 (write1), for each of the register files in the DSP core */
/* 					The purpose of this unit is to resolve a priority conflict while both ports attempt a write */
/* 					to the same register (in the same register file). */
/* 					port2 has priority over port1, thus port1 write will be denied when port2 writes to the same */
/* 					register as port1 wishes to write, at the same clock cycle. */

/*	Author: Nitzan Weinberg			*/

`define false 0

/*==============================================================================*/
/********************************************************************************/
/*																				*/
/*	module:	write_enable_selector												*/
/*																				*/
/********************************************************************************/

module write_enable_selector (
	write1_in,
	write2,
	waddr1,
	waddr2,
	write1_out
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input write1_in;
input write2;
input [1:0] waddr1;
input [1:0] waddr2;

output write1_out;


/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire write1_in;			/* write enable towards port1, generated in stage 2 in the dsp core. */
wire write2;				/* write enable towards port2, generated in stage 2 in the dsp core. */
						/* BOTH write1_in and write2 BELONG TO THE SAME REGISTER FILE !!! */
						
wire [1:0] waddr1;		/* the write address for port 1 */
wire [1:0] waddr2;		/* the write address for port 2 */

reg write1_out;			/* the resulted write enable for port1 */
						/* If both port1 and port2 tries to write the same register at the same clock cycle, */
						/* port2 has priority over port1, thus port1 write will be denied by this unit. */


/*==============================================================================*/
/*	processes																	*/
/*==============================================================================*/

always @(	write1_in or
			write2 or
			waddr1 or
			waddr2
			)
	begin
		if ( write1_in && write2 && (waddr1 == waddr2) )

			begin	/* stage 2 and stage 3 try to write to the SAME register. */
				write1_out = `false;			/* stage 2 is not allowed to write into port 1 */
			end

		else
			begin
				write1_out = write1_in;			/* coming from stage 2 */
			end
	end




endmodule	/* end module write_enable_selector */
